package com.example.switchyard.switchyard_database_binding;

import java.io.File;

import javax.ejb.EJB;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;

@RunWith(Arquillian.class)
public class SwitchyardContainerTest {
    @EJB
    private TestEntityDAO testEntityDAO;

    @Deployment
    public static JavaArchive createTestArchive() {
        JavaArchive archive = ShrinkWrap.create(JavaArchive.class, "switchyard-database-binding.jar");

        archive.addPackage(SwitchyardContainerTest.class.getPackage());

        archive.addAsResource(new File("src/main/resources/META-INF/"));

        return archive;
    }

    @Before
    public void before() {
        for (TestEntity entity : testEntityDAO.getAll()) {
            testEntityDAO.delete(entity.getId());
        }
    }

    @Test
    public void testSuccessfulEntry() {
        testEntry(new EntryPair("test", TestEntityStatus.PROCESSED));
    }


    @Test
    public void testFailureEntry() {
        testEntry(new EntryPair("fail", TestEntityStatus.FAILED));
    }

    @Test
    public void testMultipleEntry() {
        testEntry(new EntryPair("test", TestEntityStatus.PROCESSED),
                new EntryPair("fail", TestEntityStatus.FAILED),
                new EntryPair("test2", TestEntityStatus.PROCESSED));
    }

    public void testEntry(EntryPair... entries) {
        for (EntryPair entry : entries) {
            TestEntity testEntity = new TestEntity(null, entry.payload, TestEntityStatus.NEW);
            testEntity = testEntityDAO.insert(testEntity);
            entry.id = testEntity.getId();
        }

        try {
            Thread.sleep(15l * 1000l);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        for (EntryPair entry : entries) {
            TestEntity testEntity = testEntityDAO.get(entry.id);

            testEntityDAO.delete(testEntity.getId());	

            Assert.assertEquals(testEntity.getStatus(), entry.status);
        }
    }

    private class EntryPair {
        private String payload;
        private TestEntityStatus status;
        private Long id;

        EntryPair(String p, TestEntityStatus s) {
            payload = p;
            status = s;
        }
    }
}
